@@ -26,7 +26,7 @@ function build_and_install_beta() {
 
   git clone --depth 1 --no-tags "$BASHUNIT_GIT_REPO" temp_bashunit 2>/dev/null
   cd temp_bashunit
-  ./build.sh >/dev/null
+  ./build.sh bin >/dev/null
   local latest_commit=$(git rev-parse --short=7 HEAD)
   # shellcheck disable=SC2103
   cd ..
@@ -95,7 +95,7 @@ BASHUNIT_GIT_REPO="https://github.com/TypedDevs/bashunit"
 if is_git_installed; then
     LATEST_BASHUNIT_VERSION="$(get_latest_tag "$BASHUNIT_GIT_REPO")"
 else
-    LATEST_BASHUNIT_VERSION="0.23.0"
+    LATEST_BASHUNIT_VERSION="0.29.0"
 fi
 TAG="$LATEST_BASHUNIT_VERSION"
 
