@@ -5,17 +5,23 @@
 # e.g. adding custom assertions
 
 function bashunit::assertion_failed() {
+  bashunit::assert::should_skip && return 0
+
   local expected=$1
   local actual=$2
   local failure_condition_message=${3:-"but got "}
 
+  local test_fn
+  test_fn="$(bashunit::helper::find_test_function_name)"
   local label
-  label="$(helper::normalize_test_function_name "${FUNCNAME[1]}")"
-  state::add_assertions_failed
-  console_results::print_failed_test "${label}" "${expected}" \
+  label="$(bashunit::helper::normalize_test_function_name "$test_fn")"
+  bashunit::assert::mark_failed
+  bashunit::console_results::print_failed_test "${label}" "${expected}" \
     "$failure_condition_message" "${actual}"
 }
 
 function bashunit::assertion_passed() {
-  state::add_assertions_passed
+  bashunit::assert::should_skip && return 0
+
+  bashunit::state::add_assertions_passed
 }
