@@ -2,22 +2,58 @@
 
 declare -r BASHUNIT_GIT_REPO="https://github.com/TypedDevs/bashunit"
 
+#
+# Walks up the call stack to find the first function that looks like a test function.
+# A test function is one that starts with "test_" or "test" (camelCase).
+# If no test function is found, falls back to the caller of the assertion function.
+#
+# @param $1 number Optional fallback depth (default: 2, i.e., the caller of the assertion)
+#
+# @return string The test function name, or fallback function name
+#
+function bashunit::helper::find_test_function_name() {
+  local fallback_depth="${1:-2}"
+  local i
+  for ((i = 0; i < ${#FUNCNAME[@]}; i++)); do
+    local fn="${FUNCNAME[$i]}"
+    # Check if function starts with "test_" or "test" followed by uppercase
+    if [[ "$fn" == test_* ]] || [[ "$fn" =~ ^test[A-Z] ]]; then
+      echo "$fn"
+      return
+    fi
+  done
+  # No test function found, use fallback (caller of the assertion)
+  # FUNCNAME[0] = bashunit::helper::find_test_function_name
+  # FUNCNAME[1] = the assertion function (e.g., assert_same)
+  # FUNCNAME[2] = caller of the assertion
+  echo "${FUNCNAME[$fallback_depth]:-}"
+}
+
 #
 # @param $1 string Eg: "test_some_logic_camelCase"
 #
 # @return string Eg: "Some logic camelCase"
 #
-function helper::normalize_test_function_name() {
+function bashunit::helper::normalize_test_function_name() {
   local original_fn_name="${1-}"
   local interpolated_fn_name="${2-}"
 
   local custom_title
-  custom_title="$(state::get_test_title)"
+  custom_title="$(bashunit::state::get_test_title)"
   if [[ -n "$custom_title" ]]; then
     echo "$custom_title"
     return
   fi
 
+  if [[ -z "${interpolated_fn_name-}" && "${original_fn_name}" == *"::"* ]]; then
+    local state_interpolated_fn_name
+    state_interpolated_fn_name="$(bashunit::state::get_current_test_interpolated_function_name)"
+
+    if [[ -n "$state_interpolated_fn_name" ]]; then
+      interpolated_fn_name="$state_interpolated_fn_name"
+    fi
+  fi
+
   if [[ -n "${interpolated_fn_name-}" ]]; then
     original_fn_name="$interpolated_fn_name"
   fi
@@ -32,19 +68,29 @@ function helper::normalize_test_function_name() {
   fi
   # Replace underscores with spaces
   result="${result//_/ }"
-  # Capitalize the first letter
-  result="$(tr '[:lower:]' '[:upper:]' <<< "${result:0:1}")${result:1}"
+  # Capitalize the first letter (bash 3.2 compatible, no subprocess)
+  local first_char="${result:0:1}"
+  case "$first_char" in
+    a) first_char='A' ;; b) first_char='B' ;; c) first_char='C' ;; d) first_char='D' ;;
+    e) first_char='E' ;; f) first_char='F' ;; g) first_char='G' ;; h) first_char='H' ;;
+    i) first_char='I' ;; j) first_char='J' ;; k) first_char='K' ;; l) first_char='L' ;;
+    m) first_char='M' ;; n) first_char='N' ;; o) first_char='O' ;; p) first_char='P' ;;
+    q) first_char='Q' ;; r) first_char='R' ;; s) first_char='S' ;; t) first_char='T' ;;
+    u) first_char='U' ;; v) first_char='V' ;; w) first_char='W' ;; x) first_char='X' ;;
+    y) first_char='Y' ;; z) first_char='Z' ;;
+  esac
+  result="${first_char}${result:1}"
 
   echo "$result"
 }
 
-function helper::escape_single_quotes() {
+function bashunit::helper::escape_single_quotes() {
   local value="$1"
   # shellcheck disable=SC1003
   echo "${value//\'/'\'\\''\'}"
 }
 
-function helper::interpolate_function_name() {
+function bashunit::helper::interpolate_function_name() {
   local function_name="$1"
   shift
   local args=("$@")
@@ -53,7 +99,7 @@ function helper::interpolate_function_name() {
   for ((i=0; i<${#args[@]}; i++)); do
     local placeholder="::$((i+1))::"
     # shellcheck disable=SC2155
-    local value="$(helper::escape_single_quotes "${args[$i]}")"
+    local value="$(bashunit::helper::escape_single_quotes "${args[$i]}")"
     value="'$value'"
     result="${result//${placeholder}/${value}}"
   done
@@ -61,19 +107,34 @@ function helper::interpolate_function_name() {
   echo "$result"
 }
 
-function helper::decode_base64() {
+function bashunit::helper::encode_base64() {
   local value="$1"
 
   if command -v base64 >/dev/null; then
-    echo "$value" | base64 -d
+    printf '%s' "$value" | base64 -w 0 2>/dev/null || printf '%s' "$value" | base64 | tr -d '\n'
   else
-    echo "$value" | openssl enc -d -base64
+    printf '%s' "$value" | openssl enc -base64 -A
   fi
 }
 
-function helper::check_duplicate_functions() {
+function bashunit::helper::decode_base64() {
+  local value="$1"
+
+  if command -v base64 >/dev/null; then
+    printf '%s' "$value" | base64 -d
+  else
+    printf '%s' "$value" | openssl enc -d -base64
+  fi
+}
+
+function bashunit::helper::check_duplicate_functions() {
   local script="$1"
 
+  # Handle directory changes in set_up_before_script (issue #529)
+  if [[ ! -f "$script" && -n "${BASHUNIT_WORKING_DIR:-}" ]]; then
+    script="$BASHUNIT_WORKING_DIR/$script"
+  fi
+
   local filtered_lines
   filtered_lines=$(grep -E '^[[:space:]]*(function[[:space:]]+)?test[a-zA-Z_][a-zA-Z0-9_]*\s*\(\)\s*\{' "$script")
 
@@ -91,7 +152,7 @@ function helper::check_duplicate_functions() {
   local duplicates
   duplicates=$(echo "$function_names" | sort | uniq -d)
   if [ -n "$duplicates" ]; then
-    state::set_duplicated_functions_merged "$script" "$duplicates"
+    bashunit::state::set_duplicated_functions_merged "$script" "$duplicates"
     return 1
   fi
   return 0
@@ -104,7 +165,7 @@ function helper::check_duplicate_functions() {
 #
 # @return array Eg: "[prefix_filter_fn3, ...]" The filtered functions with prefix
 #
-function helper::get_functions_to_run() {
+function bashunit::helper::get_functions_to_run() {
   local prefix=$1
   local filter=${2/test_/}
   local function_names=$3
@@ -126,20 +187,25 @@ function helper::get_functions_to_run() {
 #
 # @param $1 string Eg: "do_something"
 #
-function helper::execute_function_if_exists() {
-  if [[ "$(type -t "$1")" == "function" ]]; then
-    "$1" 2>/dev/null
+function bashunit::helper::execute_function_if_exists() {
+  local fn_name="$1"
+
+  if declare -F "$fn_name" >/dev/null 2>&1; then
+    "$fn_name"
+    return $?
   fi
+
+  return 0
 }
 
 #
 # @param $1 string Eg: "do_something"
 #
-function helper::unset_if_exists() {
+function bashunit::helper::unset_if_exists() {
   unset "$1" 2>/dev/null
 }
 
-function helper::find_files_recursive() {
+function bashunit::helper::find_files_recursive() {
   ## Remove trailing slash using parameter expansion
   local path="${1%%/}"
   local pattern="${2:-*[tT]est.sh}"
@@ -166,7 +232,7 @@ function helper::find_files_recursive() {
   fi
 }
 
-function helper::normalize_variable_name() {
+function bashunit::helper::normalize_variable_name() {
   local input_string="$1"
   local normalized_string
 
@@ -179,10 +245,16 @@ function helper::normalize_variable_name() {
   echo "$normalized_string"
 }
 
-function helper::get_provider_data() {
+function bashunit::helper::get_provider_data() {
   local function_name="$1"
   local script="$2"
 
+  # Handle directory changes in set_up_before_script (issue #529)
+  # If relative path doesn't exist, try with BASHUNIT_WORKING_DIR
+  if [[ ! -f "$script" && -n "${BASHUNIT_WORKING_DIR:-}" ]]; then
+    script="$BASHUNIT_WORKING_DIR/$script"
+  fi
+
   if [[ ! -f "$script" ]]; then
     return
   fi
@@ -191,16 +263,15 @@ function helper::get_provider_data() {
   data_provider_function=$(
     # shellcheck disable=SC1087
     grep -B 2 -E "function[[:space:]]+$function_name[[:space:]]*\(\)" "$script" 2>/dev/null | \
-    grep -E "^[[:space:]]*# *@?data_provider[[:space:]]+" | \
-    sed -E 's/^[[:space:]]*# *@?data_provider[[:space:]]+//' || true
+    sed -nE 's/^[[:space:]]*# *@?data_provider[[:space:]]+//p'
   )
 
   if [[ -n "$data_provider_function" ]]; then
-    helper::execute_function_if_exists "$data_provider_function"
+    bashunit::helper::execute_function_if_exists "$data_provider_function"
   fi
 }
 
-function helper::trim() {
+function bashunit::helper::trim() {
   local input_string="$1"
   local trimmed_string
 
@@ -210,8 +281,8 @@ function helper::trim() {
   echo "$trimmed_string"
 }
 
-function helpers::get_latest_tag() {
-  if ! dependencies::has_git; then
+function bashunit::helper::get_latest_tag() {
+  if ! bashunit::dependencies::has_git; then
     return 1
   fi
 
@@ -222,7 +293,7 @@ function helpers::get_latest_tag() {
     head -n 1
 }
 
-function helpers::find_total_tests() {
+function bashunit::helper::find_total_tests() {
     local filter=${1:-}
     local files=("${@:2}")
 
@@ -246,7 +317,7 @@ function helpers::find_total_tests() {
             local all_fn_names
             all_fn_names=$(declare -F | awk '{print $3}')
             local filtered_functions
-            filtered_functions=$(helper::get_functions_to_run "test" "$filter" "$all_fn_names") || true
+            filtered_functions=$(bashunit::helper::get_functions_to_run "test" "$filter" "$all_fn_names") || true
 
             local count=0
             if [[ -n "$filtered_functions" ]]; then
@@ -257,7 +328,7 @@ function helpers::find_total_tests() {
                     local provider_data=()
                     while IFS=" " read -r line; do
                         provider_data+=("$line")
-                    done <<< "$(helper::get_provider_data "$fn_name" "$file")"
+                    done <<< "$(bashunit::helper::get_provider_data "$fn_name" "$file")"
 
                     if [[ "${#provider_data[@]}" -eq 0 ]]; then
                         count=$((count + 1))
@@ -276,7 +347,7 @@ function helpers::find_total_tests() {
     echo "$total_count"
 }
 
-function helper::load_test_files() {
+function bashunit::helper::load_test_files() {
   local filter=$1
   local files=("${@:2}")
 
@@ -286,7 +357,7 @@ function helper::load_test_files() {
     if [[ -n "${BASHUNIT_DEFAULT_PATH}" ]]; then
       while IFS='' read -r line; do
         test_files+=("$line")
-      done < <(helper::find_files_recursive "$BASHUNIT_DEFAULT_PATH")
+      done < <(bashunit::helper::find_files_recursive "$BASHUNIT_DEFAULT_PATH")
     fi
   else
     test_files=("${files[@]}")
@@ -295,7 +366,7 @@ function helper::load_test_files() {
   printf "%s\n" "${test_files[@]}"
 }
 
-function helper::load_bench_files() {
+function bashunit::helper::load_bench_files() {
   local filter=$1
   local files=("${@:2}")
 
@@ -305,7 +376,7 @@ function helper::load_bench_files() {
     if [[ -n "${BASHUNIT_DEFAULT_PATH}" ]]; then
       while IFS='' read -r line; do
         bench_files+=("$line")
-      done < <(helper::find_files_recursive "$BASHUNIT_DEFAULT_PATH" '*[bB]ench.sh')
+      done < <(bashunit::helper::find_files_recursive "$BASHUNIT_DEFAULT_PATH" '*[bB]ench.sh')
     fi
   else
     bench_files=("${files[@]}")
@@ -318,7 +389,7 @@ function helper::load_bench_files() {
 # @param $1 string function name
 # @return number line number of the function in the source file
 #
-function helper::get_function_line_number() {
+function bashunit::helper::get_function_line_number() {
   local fn_name=$1
 
   shopt -s extdebug
@@ -328,3 +399,83 @@ function helper::get_function_line_number() {
 
   echo "$line_number"
 }
+
+function bashunit::helper::generate_id() {
+  local basename="$1"
+  local sanitized_basename
+  sanitized_basename="$(bashunit::helper::normalize_variable_name "$basename")"
+  if bashunit::env::is_parallel_run_enabled; then
+    echo "${sanitized_basename}_$$_$(bashunit::random_str 6)"
+  else
+    echo "${sanitized_basename}_$$"
+  fi
+}
+
+#
+# Parses a file path that may contain a filter suffix.
+# Supports two syntaxes:
+#   - path::function_name (filter by function name)
+#   - path:line_number (filter by line number)
+#
+# @param $1 string Eg: "tests/test.sh::test_foo" or "tests/test.sh:123"
+#
+# @return string Two lines: first is file path, second is filter (or empty)
+#
+function bashunit::helper::parse_file_path_filter() {
+  local input="$1"
+  local file_path=""
+  local filter=""
+
+  # Check for :: syntax (function name filter)
+  if [[ "$input" == *"::"* ]]; then
+    file_path="${input%%::*}"
+    filter="${input#*::}"
+  # Check for :number syntax (line number filter)
+  elif [[ "$input" =~ ^(.+):([0-9]+)$ ]]; then
+    file_path="${BASH_REMATCH[1]}"
+    local line_number="${BASH_REMATCH[2]}"
+    # Line number will be resolved to function name later
+    filter="__line__:${line_number}"
+  else
+    file_path="$input"
+  fi
+
+  echo "$file_path"
+  echo "$filter"
+}
+
+#
+# Finds the test function that contains a given line number in a file.
+#
+# @param $1 string File path
+# @param $2 number Line number
+#
+# @return string The function name, or empty if not found
+#
+function bashunit::helper::find_function_at_line() {
+  local file="$1"
+  local target_line="$2"
+
+  if [[ ! -f "$file" ]]; then
+    return 1
+  fi
+
+  # Find all test function definitions and their line numbers
+  local best_match=""
+  local best_line=0
+
+  while IFS=: read -r line_num content; do
+    # Extract function name from the line
+    local fn_name=""
+    if [[ "$content" =~ ^[[:space:]]*(function[[:space:]]+)?(test[a-zA-Z_][a-zA-Z0-9_]*)[[:space:]]*\(\) ]]; then
+      fn_name="${BASH_REMATCH[2]}"
+    fi
+
+    if [[ -n "$fn_name" && "$line_num" -le "$target_line" && "$line_num" -gt "$best_line" ]]; then
+      best_match="$fn_name"
+      best_line="$line_num"
+    fi
+  done < <(grep -n -E '^[[:space:]]*(function[[:space:]]+)?test[a-zA-Z_][a-zA-Z0-9_]*[[:space:]]*\(\)' "$file")
+
+  echo "$best_match"
+}
