@@ -1,10 +1,10 @@
 #!/usr/bin/env bash
 
-function upgrade::upgrade() {
+function bashunit::upgrade::upgrade() {
   local script_path
   script_path="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
   local latest_tag
-  latest_tag="$(helpers::get_latest_tag)"
+  latest_tag="$(bashunit::helper::get_latest_tag)"
 
   if [[ "$BASHUNIT_VERSION" == "$latest_tag" ]]; then
     echo "> You are already on latest version"
@@ -14,7 +14,8 @@ function upgrade::upgrade() {
   echo "> Upgrading bashunit to latest version"
   cd "$script_path" || exit
 
-  if ! io::download_to  "https://github.com/TypedDevs/bashunit/releases/download/$latest_tag/bashunit" "bashunit"; then
+  local url="https://github.com/TypedDevs/bashunit/releases/download/$latest_tag/bashunit"
+  if ! bashunit::io::download_to "$url" "bashunit"; then
     echo "Failed to download bashunit"
   fi
 
